# python 3 headers, required if submitting to Ansible
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import argparse
import json
import requests
import time

DOCUMENTATION = """
        lookup: healthchecks
        author: Juergen Haas <juergen.haas@lakedrops.com>
        version_added: "0.9"
        short_description: sync crontabs with healthchecks platform
        description:
            - Sync a crontab spec with the healthcheck platform and returns the curl command to ping that remote item
"""


class API:

    def __init__(self, mode, base_url, api_key, channels='', host='host', user='root', jobname='placeholder',
                 schedule='* * * * *', duration=3600, disabled=False, display=None):
        self.mode = mode
        self.base_url = base_url
        self.api_key = api_key
        self.channels = channels
        self.host = host
        self.user = user
        self.jobname = jobname
        self.schedule = schedule
        self.duration = int(duration)
        self.disabled = disabled
        self.display = display
        self.headers = {
            'content-type': 'application/json',
            'X-Api-Key': api_key,
        }
        if self.duration < 60:
            self.duration = 60
        elif self.duration > 2592000:
            self.duration = 2592000

    def execute(self):
        result = 'unknown mode %s' % self.mode
        if self.mode == 'get_curl_for_ping':
            if not self.disabled:
                result = self.getcurlcommand()
                if not result:
                    result = 'echo "ERROR in crontab healthcheck setup"'
                else:
                    self.pingnew()
            else:
                self.getcurlcommand()
                result = 'disabled'
        elif self.mode == 'list':
            result = self.list()
        elif self.mode == 'ping_new':
            result = self.pingnew()
        elif self.mode == 'cleanup':
            result = self.cleanup()
        return result

    def list(self):
        checks = []
        existing = self.request('checks/')
        if existing and 'checks' in existing:
            for check in existing['checks']:
                tags = check['tags'].split(' ')
                if self.host in tags:
                    checks.append(check)
        return checks

    def pingnew(self):
        checks = []
        for check in self.list():
            if check['status'] == 'new':
                requests.get(check['ping_url'])
                checks.append(check)
        return checks

    @staticmethod
    def cleanup():
        return 'not yet implemented'

    def remove(self):
        check = self.findcheckbyname()
        if check and 'update_url' in check:
            requests.delete(check['update_url'], headers=self.headers)

    def getcurlcommand(self):
        check = self.findcheckbyname()
        if check and 'update_url' in check:
            uuid = check['update_url'].split('/')[-1]
            self.updatecheck(uuid, check)
            return 'curl -fsS --retry 5 -o /dev/null %s' % check['ping_url']
        return False

    def findcheckbyname(self, create_if_missing=True):
        existing = self.request('checks/')
        if existing and 'checks' in existing:
            for check in existing['checks']:
                if check['name'] == self.jobname:
                    tags = check['tags'].split(' ')
                    if self.host in tags and self.user in tags:
                        return check
        if not create_if_missing:
            return False
        return self.request('checks/', method='post', data={
            'name': self.jobname,
            'schedule': '* * * * *',
            'tz': 'CET',
        })

    def updatecheck(self, uuid, check):
        tags = '%s %s' % (self.host, self.user)
        if check['schedule'] != self.schedule or check['grace'] != self.duration or check['channels'] != self.channels \
                or check['tags'] != tags:
            self.request('checks/%s' % uuid, method='post', data={
                'schedule': self.schedule,
                'grace': self.duration,
                'channels': self.channels,
                'tags': tags,
            })

    def request(self, path, method='get', data=None, recurselevel=0):
        url = '%s/api/v3/%s' % (self.base_url, path)
        if self.display is not None:
            self.display.verbose(url)
        try:
            if method == 'get':
                response = requests.get(url, headers=self.headers)
                if self.display is not None:
                    self.display.verbose('Response code: %d' % response.status_code)
            elif method == 'post':
                if self.display is not None:
                    self.display.verbose(json.dumps(data))
                response = requests.post(url, headers=self.headers, data=json.dumps(data))
                if self.display is not None:
                    self.display.verbose('Response code: %d' % response.status_code)
            else:
                return {}
        except requests.RequestException as e:
            if self.display is not None:
                self.display.verbose('Failed: %s' % e)
            return {}
        if self.display is not None:
            self.display.verbose(response.content.decode('utf-8'))
        try:
            result = json.loads(response.content.decode('utf-8'))
        except json.decoder.JSONDecodeError:
            recurselevel += 1
            if recurselevel >= 3:
                return {}
            time.sleep(5 * recurselevel)
            return self.request(path, method, data, recurselevel)
        return result


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Healthcheck API')
    parser.add_argument('mode')
    parser.add_argument('url')
    parser.add_argument('apikey')
    parser.add_argument('--channels')
    parser.add_argument('--host')
    parser.add_argument('--user', default='root')
    parser.add_argument('--jobname', default='placeholder')
    parser.add_argument('--schedule', default='* * * * *')
    parser.add_argument('--duration', default=3600)
    parser.add_argument('--disabled', action='store_true')
    args = parser.parse_args()
    mode = args.mode
    url = args.url
    apikey = args.apikey
    args.__delattr__('mode')
    args.__delattr__('url')
    args.__delattr__('apikey')
    api = API(mode, url, apikey, **args.__dict__)
    print(api.execute())
else:
    from ansible.plugins.lookup import LookupBase

    try:
        from __main__ import display
    except ImportError:
        from ansible.utils.display import Display
        display = Display()


    class LookupModule(LookupBase):

        def run(self, dummy, mode='get_curl_for_ping', user='root', jobname='placeholder', schedule='* * * * *',
                duration=3600, disabled=False, **kwargs):
            variables = getattr(self._templar, '_available_variables', {})
            base_url = 'https://healthchecks.lakedrops.com'
            api_key = '%s' % variables['healthchecks']['api']
            channels = ','.join(variables['healthchecks']['channels'])
            host = variables['inventory_hostname']
            hcapi = API(mode, base_url, api_key, channels=channels, host=host, user=user, jobname=jobname,
                        schedule=schedule, duration=duration, disabled=disabled, display=display)
            return [hcapi.execute()]
